@@ -21,14 +21,15 @@
 //!
 //! Manages and stores the Scoring logic of a particular peer on the gossipsub behaviour.
 
+use crate::metrics::{Metrics, Penalty};
 use crate::time_cache::TimeCache;
 use crate::{MessageId, TopicHash};
-use instant::Instant;
 use libp2p_core::PeerId;
 use log::{debug, trace, warn};
 use std::collections::{hash_map, HashMap, HashSet};
 use std::net::IpAddr;
 use std::time::Duration;
+use wasm_timer::Instant;
 
 mod params;
 use crate::error::ValidationError;
@@ -212,8 +213,14 @@ impl PeerScore {
         }
     }
 
-    /// Returns the score for a peer.
+    /// Returns the score for a peer
     pub fn score(&self, peer_id: &PeerId) -> f64 {
+        self.metric_score(peer_id, None)
+    }
+
+    /// Returns the score for a peer, logging metrics. This is called from the heartbeat and
+    /// increments the metric counts for penalties.
+    pub fn metric_score(&self, peer_id: &PeerId, mut metrics: Option<&mut Metrics>) -> f64 {
         let peer_stats = match self.peer_stats.get(peer_id) {
             Some(v) => v,
             None => return 0.0,
@@ -264,8 +271,11 @@ impl PeerScore {
                         - topic_stats.mesh_message_deliveries;
                     let p3 = deficit * deficit;
                     topic_score += p3 * topic_params.mesh_message_deliveries_weight;
+                    if let Some(metrics) = metrics.as_mut() {
+                        metrics.register_score_penalty(Penalty::MessageDeficit);
+                    }
                     debug!(
-                        "The peer {} has a mesh message deliveries deficit of {} in topic\
+                        "[Penalty] The peer {} has a mesh message deliveries deficit of {} in topic\
                          {} and will get penalized by {}",
                         peer_id,
                         deficit,
@@ -313,8 +323,11 @@ impl PeerScore {
                 if (peers_in_ip as f64) > self.params.ip_colocation_factor_threshold {
                     let surplus = (peers_in_ip as f64) - self.params.ip_colocation_factor_threshold;
                     let p6 = surplus * surplus;
+                    if let Some(metrics) = metrics.as_mut() {
+                        metrics.register_score_penalty(Penalty::IPColocation);
+                    }
                     debug!(
-                        "The peer {} gets penalized because of too many peers with the ip {}. \
+                        "[Penalty] The peer {} gets penalized because of too many peers with the ip {}. \
                         The surplus is {}. ",
                         peer_id, ip, surplus
                     );
@@ -335,7 +348,7 @@ impl PeerScore {
     pub fn add_penalty(&mut self, peer_id: &PeerId, count: usize) {
         if let Some(peer_stats) = self.peer_stats.get_mut(peer_id) {
             debug!(
-                "Behavioral penalty for peer {}, count = {}.",
+                "[Penalty] Behavioral penalty for peer {}, count = {}.",
                 peer_id, count
             );
             peer_stats.behaviour_penalty += count as f64;
@@ -597,9 +610,10 @@ impl PeerScore {
     /// Similar to `reject_message` except does not require the message id or reason for an invalid message.
     pub fn reject_invalid_message(&mut self, from: &PeerId, topic_hash: &TopicHash) {
         debug!(
-            "Message from {} rejected because of ValidationError or SelfOrigin",
+            "[Penalty] Message from {} rejected because of ValidationError or SelfOrigin",
             from
         );
+
         self.mark_invalid_message_delivery(from, topic_hash);
     }
 
@@ -630,9 +644,9 @@ impl PeerScore {
                 .entry(msg_id.clone())
                 .or_insert_with(DeliveryRecord::default);
 
-            // this should be the first delivery trace
+            // Multiple peers can now reject the same message as we track which peers send us the
+            // message. If we have already updated the status, return.
             if record.status != DeliveryStatus::Unknown {
-                warn!("Unexpected delivery trace: Message from {} was first seen {}s ago and has a delivery status {:?}", from, record.first_seen.elapsed().as_secs(), record.status);
                 return;
             }
 
@@ -764,7 +778,7 @@ impl PeerScore {
                 peer_stats.stats_or_default_mut(topic_hash.clone(), &self.params)
             {
                 debug!(
-                    "Peer {} delivered an invalid message in topic {} and gets penalized \
+                    "[Penalty] Peer {} delivered an invalid message in topic {} and gets penalized \
                     for it",
                     peer_id, topic_hash
                 );
